@@ -268,6 +268,16 @@ string IIS7Request::getRemoteAddr() const
 
 string IIS7Request::getSecureHeader(const char* name) const
 {
+    if (m_useVariables) {
+        PCWSTR p;
+        DWORD len;
+        HRESULT hr = m_ctx->GetServerVariable(name, &p, &len);
+        if (SUCCEEDED(hr) && p) {
+            std::wstring_convert<std::codecvt_utf8_utf16<wchar_t>> converter;
+            return converter.to_bytes(p);
+        }
+        return "";
+    }
     PCSTR p = m_request->GetHeader(g_bSafeHeaderNames ? makeSafeHeader(name).c_str() : name);
     return (nullptr == p) ? "" : p;
 }
@@ -419,6 +429,8 @@ void IIS7Request::setResponseHeader(const char* name, const char* value, bool re
 long IIS7Request::sendRedirect(const char* url)
 {
     HTTPResponse::sendRedirect(url);
+    setResponseHeader("Expires", "Wed, 01 Jan 1997 12:00:00 GMT", true);
+    setResponseHeader("Cache-Control", "private,no-store,no-cache,max-age=0", true);
     HRESULT hr = m_response->Redirect(url);
     if (FAILED(hr)) {
         logFatal("Redirect", hr);
