@@ -46,6 +46,8 @@
 #include <xmltooling/util/NDC.h>
 #include <xmltooling/util/XMLConstants.h>
 #include <xmltooling/util/XMLHelper.h>
+#include <xmltooling/logging.h>
+
 #include <xercesc/util/Base64.hpp>
 #include <xercesc/util/XMLUniDefs.hpp>
 
@@ -59,6 +61,9 @@ using namespace xercesc;
 using namespace boost;
 using namespace std;
 
+using xmltooling::logging::Category;
+using xmltooling::logging::Priority;
+
 // globals
 namespace {
     static const XMLCh path[] =             UNICODE_LITERAL_4(p,a,t,h);
@@ -68,6 +73,8 @@ namespace {
     static const XMLCh sslport[] =          UNICODE_LITERAL_7(s,s,l,p,o,r,t);
     static const XMLCh scheme[] =           UNICODE_LITERAL_6(s,c,h,e,m,e);
     static const XMLCh id[] =               UNICODE_LITERAL_2(i,d);
+    static const XMLCh useHeaders[] =       UNICODE_LITERAL_10(u, s, e, H, e, a, d, e, r, s);
+    static const XMLCh useVariables[] =     UNICODE_LITERAL_12(u, s, e, V, a, r, i, a, b, l, e, s);
     static const XMLCh Alias[] =            UNICODE_LITERAL_5(A,l,i,a,s);
     static const XMLCh Site[] =             UNICODE_LITERAL_4(S,i,t,e);
 
@@ -81,7 +88,7 @@ namespace {
             e = XMLHelper::getFirstChildElement(e, Alias);
             while (e) {
                 if (e->hasChildNodes()) {
-                    auto_ptr_char alias(e->getTextContent());
+                    auto_ptr_char alias(XMLHelper::getTextContent(e));
                     m_aliases.insert(alias.get());
                 }
                 e = XMLHelper::getNextSiblingElement(e, Alias);
@@ -102,20 +109,6 @@ namespace {
     vector<string> g_NoCerts;
 }
 
-BOOL LogEvent(
-    LPCSTR  lpUNCServerName,
-    WORD  wType,
-    DWORD  dwEventID,
-    PSID  lpUserSid,
-    LPCSTR  message)
-{
-    LPCSTR  messages[] = {message, nullptr};
-
-    HANDLE hElog = RegisterEventSource(lpUNCServerName, "Shibboleth ISAPI Filter");
-    BOOL res = ReportEvent(hElog, wType, 0, dwEventID, lpUserSid, 1, 0, messages, nullptr);
-    return (DeregisterEventSource(hElog) && res);
-}
-
 void _my_invalid_parameter_handler(
    const wchar_t * expression,
    const wchar_t * function,
@@ -140,13 +133,12 @@ extern "C" BOOL WINAPI GetExtensionVersion(HSE_VERSION_INFO* pVer)
         return FALSE;
 
     if (!g_Config) {
-        LogEvent(nullptr, EVENTLOG_ERROR_TYPE, 2100, nullptr,
-                "Extension mode startup not possible, is the DLL loaded as a filter?");
+        Category::getInstance(SHIBSP_LOGCAT ".ISAPI").fatal("extension mode startup not possible, is the DLL loaded as a filter?");
         return FALSE;
     }
 
     pVer->dwExtensionVersion = HSE_VERSION;
-    strncpy(pVer->lpszExtensionDesc,"Shibboleth ISAPI Extension",HSE_MAX_EXT_DLL_NAME_LEN-1);
+    strncpy(pVer->lpszExtensionDesc, "Shibboleth ISAPI Extension", HSE_MAX_EXT_DLL_NAME_LEN-1);
     return TRUE;
 }
 
@@ -157,15 +149,17 @@ extern "C" BOOL WINAPI TerminateExtension(DWORD)
 
 extern "C" BOOL WINAPI GetFilterVersion(PHTTP_FILTER_VERSION pVer)
 {
+    Category& log = Category::getInstance(SHIBSP_LOGCAT ".ISAPI");
+
     if (!pVer)
         return FALSE;
     else if (g_Config) {
-        LogEvent(nullptr, EVENTLOG_ERROR_TYPE, 2100, nullptr,
-                "Reentrant filter initialization, ignoring...");
+        log.warn("reentrant ISAPI filter initialization, ignoring...");
         return TRUE;
     }
 
     g_Config = &SPConfig::getConfig();
+    g_Config->deprecation().warn("ISAPI extension is replaced by IIS 7+ module");
     g_Config->setFeatures(
         SPConfig::Listener |
         SPConfig::Caching |
@@ -176,8 +170,7 @@ extern "C" BOOL WINAPI GetFilterVersion(PHTTP_FILTER_VERSION pVer)
         );
     if (!g_Config->init()) {
         g_Config = nullptr;
-        LogEvent(nullptr, EVENTLOG_ERROR_TYPE, 2100, nullptr,
-                "Filter startup failed during library initialization, check native log for help.");
+        log.fatal("ISAPI filter startup failed during library initialization, check native log for help");
         return FALSE;
     }
 
@@ -185,12 +178,10 @@ extern "C" BOOL WINAPI GetFilterVersion(PHTTP_FILTER_VERSION pVer)
         if (!g_Config->instantiate(nullptr, true))
             throw runtime_error("unknown error");
     }
-    catch (std::exception& ex) {
+    catch (const std::exception& ex) {
+        log.fatal("ISAPI filter startup failed: %s", ex.what());
         g_Config->term();
         g_Config=nullptr;
-        LogEvent(nullptr, EVENTLOG_ERROR_TYPE, 2100, nullptr, ex.what());
-        LogEvent(nullptr, EVENTLOG_ERROR_TYPE, 2100, nullptr,
-                "Filter startup failed to load configuration, check native log for details.");
         return FALSE;
     }
 
@@ -221,8 +212,7 @@ extern "C" BOOL WINAPI GetFilterVersion(PHTTP_FILTER_VERSION pVer)
                 }
                 else {
                     _set_invalid_parameter_handler(old);
-                    LogEvent(nullptr, EVENTLOG_ERROR_TYPE, 2100, nullptr,
-                            "Filter failed to generate a random anti-spoofing key (if this is Windows 2000 set one manually).");
+                    log.fatal("ISAPI filter failed to generate a random anti-spoofing key");
                     locker.assign();    // pops lock on SP config
                     g_Config->term();
                     g_Config = nullptr;
@@ -237,13 +227,26 @@ extern "C" BOOL WINAPI GetFilterVersion(PHTTP_FILTER_VERSION pVer)
             g_bNormalizeRequest = !flag.first || flag.second;
             flag = props->getBool("safeHeaderNames");
             g_bSafeHeaderNames = flag.first && flag.second;
+            if (props->getString("useHeaders").first)
+                log.warn("useHeaders attribute not supported by ISAPI filter, ignored");
+            if (props->getString("useVariables").first)
+                log.warn("useVariables attribute not supported by ISAPI filter, ignored");
+
             const DOMElement* child = XMLHelper::getFirstChildElement(props->getElement(), Site);
             while (child) {
                 string id(XMLHelper::getAttrString(child, "", id));
-                if (!id.empty())
+                if (!id.empty()) {
                     g_Sites.insert(make_pair(id, site_t(child)));
+                    if (!XMLHelper::getAttrString(child, "", useHeaders).empty())
+                        log.warn("useHeaders attribute not valid for this filter");
+                    if (!XMLHelper::getAttrString(child, "", useVariables).empty())
+                        log.warn("useVariables attribute not valid for this filter");
+                }
                 child = XMLHelper::getNextSiblingElement(child, Site);
             }
+
+            if (nullptr != props->getPropertySet("Roles"))
+                log.warn("<Roles> element not valid for this filter");
         }
     }
 
@@ -254,7 +257,7 @@ extern "C" BOOL WINAPI GetFilterVersion(PHTTP_FILTER_VERSION pVer)
                    SF_NOTIFY_NONSECURE_PORT |
                    SF_NOTIFY_PREPROC_HEADERS |
                    SF_NOTIFY_LOG);
-    LogEvent(nullptr, EVENTLOG_INFORMATION_TYPE, 7701, nullptr, "Filter initialized...");
+    log.info("ISAPI filter initialized");
     return TRUE;
 }
 
@@ -263,7 +266,7 @@ extern "C" BOOL WINAPI TerminateFilter(DWORD)
     if (g_Config)
         g_Config->term();
     g_Config = nullptr;
-    LogEvent(nullptr, EVENTLOG_INFORMATION_TYPE, 7701, nullptr, "Filter shut down...");
+    Category::getInstance(SHIBSP_LOGCAT ".ISAPI").info("ISAPI filter shutting down");
     return TRUE;
 }
 
@@ -333,16 +336,21 @@ class ShibTargetIsapiF : public AbstractSPRequest
 
 public:
   ShibTargetIsapiF(PHTTP_FILTER_CONTEXT pfc, PHTTP_FILTER_PREPROC_HEADERS pn, const site_t& site)
-      : AbstractSPRequest(SHIBSP_LOGCAT".ISAPI"), m_pfc(pfc), m_pn(pn), m_allhttp(4096), m_firsttime(true) {
+      : AbstractSPRequest(SHIBSP_LOGCAT ".ISAPI"), m_pfc(pfc), m_pn(pn), m_allhttp(4096), m_firsttime(true) {
+
+    static char _url[] = "url";
+    static char _SERVER_PORT[] = "SERVER_PORT";
+    static char _SERVER_NAME[] = "SERVER_NAME";
+    static char _ShibSpoofCheck[] = "ShibSpoofCheck:";
 
     // URL path always come from IIS.
     dynabuf var(256);
-    GetHeader("url",var,256,false);
+    GetHeader(_url,var,256,false);
     setRequestURI(var);
 
     // Port may come from IIS or from site def.
     if (!g_bNormalizeRequest || (pfc->fIsSecurePort && site.m_sslport.empty()) || (!pfc->fIsSecurePort && site.m_port.empty())) {
-        GetServerVariable("SERVER_PORT",var,10);
+        GetServerVariable(_SERVER_PORT,var,10);
         if (var.empty()) {
             m_port = pfc->fIsSecurePort ? 443 : 80;
         }
@@ -362,7 +370,7 @@ public:
     if (m_scheme.empty() || !g_bNormalizeRequest)
         m_scheme=pfc->fIsSecurePort ? "https" : "http";
 
-    GetServerVariable("SERVER_NAME",var,32);
+    GetServerVariable(_SERVER_NAME,var,32);
 
     // Make sure SERVER_NAME is "authorized" for use on this site. If not, or empty, set to canonical name.
     if (var.empty()) {
@@ -375,7 +383,7 @@ public:
     }
 
     if (!g_spoofKey.empty()) {
-        GetHeader("ShibSpoofCheck:", var, 32, false);
+        GetHeader(_ShibSpoofCheck, var, 32, false);
         if (!var.empty() && g_spoofKey == (char*)var)
             m_firsttime = false;
     }
@@ -400,38 +408,36 @@ public:
       return uri ? (uri + 1) : nullptr;
   }
   const char* getMethod() const {
+    static char _HTTP_METHOD[] = "HTTP_METHOD";
     if (m_method.empty()) {
         dynabuf var(5);
-        GetServerVariable("HTTP_METHOD",var,5,false);
+        GetServerVariable(_HTTP_METHOD,var,5,false);
         if (!var.empty())
             m_method = var;
     }
     return m_method.c_str();
   }
   string getContentType() const {
+    static char _HTTP_CONTENT_TYPE[] = "HTTP_CONTENT_TYPE";
     if (m_content_type.empty()) {
         dynabuf var(32);
-        GetServerVariable("HTTP_CONTENT_TYPE",var,32,false);
+        GetServerVariable(_HTTP_CONTENT_TYPE,var,32,false);
         if (!var.empty())
             m_content_type = var;
     }
     return m_content_type;
   }
   string getRemoteAddr() const {
+    static char _REMOTE_ADDR[] = "REMOTE_ADDR";
     m_remote_addr = AbstractSPRequest::getRemoteAddr();
     if (m_remote_addr.empty()) {
         dynabuf var(16);
-        GetServerVariable("REMOTE_ADDR",var,16,false);
+        GetServerVariable(_REMOTE_ADDR,var,16,false);
         if (!var.empty())
             m_remote_addr = var;
     }
     return m_remote_addr;
   }
-  void log(SPLogLevel level, const string& msg) const {
-    AbstractSPRequest::log(level,msg);
-    if (level >= SPCrit)
-        LogEvent(nullptr, EVENTLOG_ERROR_TYPE, 2100, nullptr, msg.c_str());
-  }
   string makeSafeHeader(const char* rawname) const {
       string hdr;
       for (; *rawname; ++rawname) {
@@ -441,9 +447,13 @@ public:
       return (hdr + ':');
   }
   void clearHeader(const char* rawname, const char* cginame) {
+    static char _ALL_HTTP[] = "ALL_HTTP";
+    static char _REMOTE_USER[] = "remote-user:";
+    static char _REMOTE_USER2[] = "remote_user:";
+
     if (g_checkSpoofing && m_firsttime) {
         if (m_allhttp.empty())
-	        GetServerVariable( "ALL_HTTP", m_allhttp, 4096, false);
+	        GetServerVariable(_ALL_HTTP, m_allhttp, 4096, false);
         if (!m_allhttp.empty()) {
             string hdr = g_bSafeHeaderNames ? ("HTTP_" + makeSafeHeader(cginame + 5)) : (string(cginame) + ':');
             if (strstr(m_allhttp, hdr.c_str()))
@@ -455,12 +465,12 @@ public:
         m_pn->SetHeader(m_pfc, const_cast<char*>(hdr.c_str()), const_cast<char*>(g_unsetHeaderValue.c_str()));
     }
     else if (!strcmp(rawname,"REMOTE_USER")) {
-	    m_pn->SetHeader(m_pfc, "remote-user:", const_cast<char*>(g_unsetHeaderValue.c_str()));
-        m_pn->SetHeader(m_pfc, "remote_user:", const_cast<char*>(g_unsetHeaderValue.c_str()));
+        m_pn->SetHeader(m_pfc, _REMOTE_USER, const_cast<char*>(g_unsetHeaderValue.c_str()));
+        m_pn->SetHeader(m_pfc, _REMOTE_USER2, const_cast<char*>(g_unsetHeaderValue.c_str()));
 	}
 	else {
-	    string hdr = string(rawname) + ':';
-	    m_pn->SetHeader(m_pfc, const_cast<char*>(hdr.c_str()), const_cast<char*>(g_unsetHeaderValue.c_str()));
+        string hdr = string(rawname) + ':';
+        m_pn->SetHeader(m_pfc, const_cast<char*>(hdr.c_str()), const_cast<char*>(g_unsetHeaderValue.c_str()));
 	}
   }
   void setHeader(const char* name, const char* value) {
@@ -471,14 +481,14 @@ public:
     string hdr = g_bSafeHeaderNames ? makeSafeHeader(name) : (string(name) + ':');
     dynabuf buf(256);
     GetHeader(const_cast<char*>(hdr.c_str()), buf, 256, false);
-    return string(buf.empty() ? "" : buf);
+    return string(buf.empty() ? "" : static_cast<char*>(buf));
   }
   string getHeader(const char* name) const {
     string hdr(name);
     hdr += ':';
     dynabuf buf(256);
     GetHeader(const_cast<char*>(hdr.c_str()), buf, 256, false);
-    return string(buf.empty() ? "" : buf);
+    return string(buf.empty() ? "" : static_cast<char*>(buf));
   }
   void setRemoteUser(const char* user) {
     setHeader("remote-user", user);
@@ -490,13 +500,15 @@ public:
   string getRemoteUser() const {
     return getSecureHeader("remote-user");
   }
-  void setResponseHeader(const char* name, const char* value) {
-    HTTPResponse::setResponseHeader(name, value);
-    // Set for later.
-    if (value)
-        m_headers.insert(make_pair(name,value));
-    else
-        m_headers.erase(name);
+  void setResponseHeader(const char* name, const char* value, bool replace=false) {
+    HTTPResponse::setResponseHeader(name, value, replace);
+    if (name && *name) {
+        // Set for later.
+        if (replace || !value)
+            m_headers.erase(name);
+        if (value && *value)
+            m_headers.insert(make_pair(name,value));
+    }
   }
   long sendResponse(istream& in, long status) {
     string hdr = string("Connection: close\r\n");
@@ -521,6 +533,7 @@ public:
     return SF_STATUS_REQ_FINISHED;
   }
   long sendRedirect(const char* url) {
+    static char _status[] = "302 Please Wait";
     HTTPResponse::sendRedirect(url);
     string hdr=string("Location: ") + url + "\r\n"
       "Content-Type: text/html\r\n"
@@ -530,7 +543,7 @@ public:
     for (multimap<string,string>::const_iterator i = m_headers.begin(); i != m_headers.end(); ++i)
         hdr += i->first + ": " + i->second + "\r\n";
     hdr += "\r\n";
-    m_pfc->ServerSupportFunction(m_pfc, SF_REQ_SEND_RESPONSE_HEADER, "302 Please Wait", (ULONG_PTR)hdr.c_str(), 0);
+    m_pfc->ServerSupportFunction(m_pfc, SF_REQ_SEND_RESPONSE_HEADER, _status, (ULONG_PTR)hdr.c_str(), 0);
     static const char* redmsg="<HTML><BODY>Redirecting...</BODY></HTML>";
     DWORD resplen=40;
     m_pfc->WriteClient(m_pfc, (LPVOID)redmsg, &resplen, 0);
@@ -588,18 +601,18 @@ public:
 
 DWORD WriteClientError(PHTTP_FILTER_CONTEXT pfc, const char* msg)
 {
-    LogEvent(nullptr, EVENTLOG_ERROR_TYPE, 2100, nullptr, msg);
-    static const char* ctype="Connection: close\r\nContent-Type: text/html\r\n\r\n";
-    pfc->ServerSupportFunction(pfc,SF_REQ_SEND_RESPONSE_HEADER,"200 OK",(ULONG_PTR)ctype,0);
-    static const char* xmsg="<HTML><HEAD><TITLE>Shibboleth Filter Error</TITLE></HEAD><BODY>"
+    static char _status[] = "200 OK";
+    static char ctype[] = "Connection: close\r\nContent-Type: text/html\r\n\r\n";
+    pfc->ServerSupportFunction(pfc,SF_REQ_SEND_RESPONSE_HEADER,_status,(ULONG_PTR)ctype,0);
+    static char xmsg[] = "<HTML><HEAD><TITLE>Shibboleth Filter Error</TITLE></HEAD><BODY>"
                             "<H1>Shibboleth Filter Error</H1>";
     DWORD resplen=strlen(xmsg);
-    pfc->WriteClient(pfc,(LPVOID)xmsg,&resplen,0);
+    pfc->WriteClient(pfc,xmsg,&resplen,0);
     resplen=strlen(msg);
-    pfc->WriteClient(pfc,(LPVOID)msg,&resplen,0);
-    static const char* xmsg2="</BODY></HTML>";
+    pfc->WriteClient(pfc,const_cast<char*>(msg),&resplen,0);
+    static char xmsg2[] = "</BODY></HTML>";
     resplen=strlen(xmsg2);
-    pfc->WriteClient(pfc,(LPVOID)xmsg2,&resplen,0);
+    pfc->WriteClient(pfc,xmsg2,&resplen,0);
     return SF_STATUS_REQ_FINISHED;
 }
 
@@ -618,14 +631,16 @@ void GetServerVariable(PHTTP_FILTER_CONTEXT pfc, LPSTR lpszVariable, dynabuf& s,
             break;
     }
     if (bRequired && s.empty()) {
-        string msg = string("Missing required server variable: ") + lpszVariable;
-        LogEvent(nullptr, EVENTLOG_ERROR_TYPE, 2100, nullptr, msg.c_str());
+        Category::getInstance(SHIBSP_LOGCAT ".ISAPI").error("missing server variable: %s", lpszVariable);
     }
 }
 
 
 extern "C" DWORD WINAPI HttpFilterProc(PHTTP_FILTER_CONTEXT pfc, DWORD notificationType, LPVOID pvNotification)
 {
+    static char _INSTANCE_ID[] = "INSTANCE_ID";
+    static char _ShibSpoofCheck[] = "ShibSpoofCheck:";
+
     // Is this a log notification?
     if (notificationType == SF_NOTIFY_LOG) {
         if (pfc->pFilterContext)
@@ -637,7 +652,7 @@ extern "C" DWORD WINAPI HttpFilterProc(PHTTP_FILTER_CONTEXT pfc, DWORD notificat
     try {
         // Determine web site number. This can't really fail, I don't think.
         dynabuf buf(128);
-        GetServerVariable(pfc,"INSTANCE_ID",buf,10);
+        GetServerVariable(pfc,_INSTANCE_ID,buf,10);
         if (buf.empty())
             return WriteClientError(pfc, "Shibboleth Filter failed to obtain INSTANCE_ID server variable.");
 
@@ -654,7 +669,7 @@ extern "C" DWORD WINAPI HttpFilterProc(PHTTP_FILTER_CONTEXT pfc, DWORD notificat
 
         pair<bool,long> res = stf.getServiceProvider().doAuthentication(stf);
         if (!g_spoofKey.empty())
-            pn->SetHeader(pfc, "ShibSpoofCheck:", const_cast<char*>(g_spoofKey.c_str()));
+            pn->SetHeader(pfc, _ShibSpoofCheck, const_cast<char*>(g_spoofKey.c_str()));
         if (res.first) return res.second;
 
         res = stf.getServiceProvider().doExport(stf);
@@ -665,7 +680,7 @@ extern "C" DWORD WINAPI HttpFilterProc(PHTTP_FILTER_CONTEXT pfc, DWORD notificat
 
         return SF_STATUS_REQ_NEXT_NOTIFICATION;
     }
-    catch(bad_alloc) {
+    catch(const bad_alloc&) {
         return WriteClientError(pfc, "Out of Memory");
     }
     catch(long e) {
@@ -674,17 +689,16 @@ extern "C" DWORD WINAPI HttpFilterProc(PHTTP_FILTER_CONTEXT pfc, DWORD notificat
         else
             return WriteClientError(pfc, "Shibboleth Filter detected unexpected IIS error.");
     }
-    catch (std::exception& e) {
-        LogEvent(nullptr, EVENTLOG_ERROR_TYPE, 2100, nullptr, e.what());
+    catch (const std::exception& e) {
+        Category::getInstance(SHIBSP_LOGCAT ".ISAPI").error("ISAPI filter caught an exception: %s", e.what());
         return WriteClientError(pfc, "Shibboleth Filter caught an exception, check Event Log for details.");
     }
     catch(...) {
-        LogEvent(nullptr, EVENTLOG_ERROR_TYPE, 2100, nullptr, "Shibboleth Filter threw an unknown exception.");
+        Category::getInstance(SHIBSP_LOGCAT ".ISAPI").crit("ISAPI extension caught an unknown exception");
         if (g_catchAll)
             return WriteClientError(pfc, "Shibboleth Filter threw an unknown exception.");
         throw;
     }
-
     return WriteClientError(pfc, "Shibboleth Filter reached unreachable code, save my walrus!");
 }
 
@@ -694,17 +708,17 @@ extern "C" DWORD WINAPI HttpFilterProc(PHTTP_FILTER_CONTEXT pfc, DWORD notificat
 
 DWORD WriteClientError(LPEXTENSION_CONTROL_BLOCK lpECB, const char* msg)
 {
-    LogEvent(nullptr, EVENTLOG_ERROR_TYPE, 2100, nullptr, msg);
-    static const char* ctype="Connection: close\r\nContent-Type: text/html\r\n\r\n";
-    lpECB->ServerSupportFunction(lpECB->ConnID,HSE_REQ_SEND_RESPONSE_HEADER,"200 OK",0,(LPDWORD)ctype);
-    static const char* xmsg="<HTML><HEAD><TITLE>Shibboleth Error</TITLE></HEAD><BODY><H1>Shibboleth Error</H1>";
+    static char _status[] = "200 OK";
+    static char ctype[] = "Connection: close\r\nContent-Type: text/html\r\n\r\n";
+    lpECB->ServerSupportFunction(lpECB->ConnID,HSE_REQ_SEND_RESPONSE_HEADER,_status,0,(LPDWORD)ctype);
+    static char xmsg[] = "<HTML><HEAD><TITLE>Shibboleth Error</TITLE></HEAD><BODY><H1>Shibboleth Error</H1>";
     DWORD resplen=strlen(xmsg);
-    lpECB->WriteClient(lpECB->ConnID,(LPVOID)xmsg,&resplen,HSE_IO_SYNC);
+    lpECB->WriteClient(lpECB->ConnID,xmsg,&resplen,HSE_IO_SYNC);
     resplen=strlen(msg);
-    lpECB->WriteClient(lpECB->ConnID,(LPVOID)msg,&resplen,HSE_IO_SYNC);
-    static const char* xmsg2="</BODY></HTML>";
+    lpECB->WriteClient(lpECB->ConnID,const_cast<char*>(msg),&resplen,HSE_IO_SYNC);
+    static char xmsg2[] = "</BODY></HTML>";
     resplen=strlen(xmsg2);
-    lpECB->WriteClient(lpECB->ConnID,(LPVOID)xmsg2,&resplen,HSE_IO_SYNC);
+    lpECB->WriteClient(lpECB->ConnID,xmsg2,&resplen,HSE_IO_SYNC);
     return HSE_STATUS_SUCCESS;
 }
 
@@ -722,9 +736,14 @@ class ShibTargetIsapiE : public AbstractSPRequest
 
 public:
   ShibTargetIsapiE(LPEXTENSION_CONTROL_BLOCK lpECB, const site_t& site)
-      : AbstractSPRequest(SHIBSP_LOGCAT".ISAPI"), m_lpECB(lpECB), m_gotBody(false) {
+      : AbstractSPRequest(SHIBSP_LOGCAT ".ISAPI"), m_lpECB(lpECB), m_gotBody(false) {
+    static char _HTTPS[] = "HTTPS";
+    static char _URL[] = "URL";
+    static char _SERVER_PORT[] = "SERVER_PORT";
+    static char _SERVER_NAME[] = "SERVER_NAME";
+
     dynabuf ssl(5);
-    GetServerVariable("HTTPS",ssl,5);
+    GetServerVariable(_HTTPS,ssl,5);
     bool SSL=(ssl=="on" || ssl=="ON");
 
     // Scheme may come from site def or be derived from IIS.
@@ -734,12 +753,12 @@ public:
 
     // URL path always come from IIS.
     dynabuf url(256);
-    GetServerVariable("URL",url,255);
+    GetServerVariable(_URL,url,255);
 
     // Port may come from IIS or from site def.
     if (!g_bNormalizeRequest || (SSL && site.m_sslport.empty()) || (!SSL && site.m_port.empty())) {
         dynabuf port(11);
-        GetServerVariable("SERVER_PORT",port,10);
+        GetServerVariable(_SERVER_PORT,port,10);
         if (port.empty()) {
             m_port = SSL ? 443 : 80;
         }
@@ -755,7 +774,7 @@ public:
     }
 
     dynabuf var(32);
-    GetServerVariable("SERVER_NAME", var, 32);
+    GetServerVariable(_SERVER_NAME, var, 32);
     if (var.empty()) {
         m_hostname = site.m_name;
     }
@@ -829,29 +848,26 @@ public:
       return m_lpECB->cbTotalBytes;
   }
   string getRemoteUser() const {
+    static char _REMOTE_USER[] = "REMOTE_USER";
     if (m_remote_user.empty()) {
         dynabuf var(16);
-        GetServerVariable("REMOTE_USER", var, 32, false);
+        GetServerVariable(_REMOTE_USER, var, 32, false);
         if (!var.empty())
             m_remote_user = var;
     }
     return m_remote_user;
   }
   string getRemoteAddr() const {
+      static char _REMOTE_ADDR[] = "REMOTE_ADDR";
     m_remote_addr = AbstractSPRequest::getRemoteAddr();
     if (m_remote_addr.empty()) {
         dynabuf var(16);
-        GetServerVariable("REMOTE_ADDR", var, 16, false);
+        GetServerVariable(_REMOTE_ADDR, var, 16, false);
         if (!var.empty())
             m_remote_addr = var;
     }
     return m_remote_addr;
   }
-  void log(SPLogLevel level, const string& msg) const {
-      AbstractSPRequest::log(level,msg);
-      if (level >= SPCrit)
-          LogEvent(nullptr, EVENTLOG_ERROR_TYPE, 2100, nullptr, msg.c_str());
-  }
   string getHeader(const char* name) const {
     string hdr("HTTP_");
     for (; *name; ++name) {
@@ -862,15 +878,17 @@ public:
     }
     dynabuf buf(128);
     GetServerVariable(const_cast<char*>(hdr.c_str()), buf, 128, false);
-    return buf.empty() ? "" : buf;
+    return buf.empty() ? "" : static_cast<char*>(buf);
   }
-  void setResponseHeader(const char* name, const char* value) {
-    HTTPResponse::setResponseHeader(name, value);
-    // Set for later.
-    if (value)
-        m_headers.insert(make_pair(name,value));
-    else
-        m_headers.erase(name);
+  void setResponseHeader(const char* name, const char* value, bool replace = false) {
+      HTTPResponse::setResponseHeader(name, value, replace);
+      if (name && *name) {
+          // Set for later.
+          if (replace || !value)
+              m_headers.erase(name);
+          if (value && *value)
+              m_headers.insert(make_pair(name, value));
+      }
   }
   const char* getQueryString() const {
     return m_lpECB->lpszQueryString;
@@ -913,15 +931,26 @@ public:
     for (multimap<string,string>::const_iterator i = m_headers.begin(); i != m_headers.end(); ++i)
         hdr += i->first + ": " + i->second + "\r\n";
     hdr += "\r\n";
-    const char* codestr="200 OK";
+
+    static char okstr[] = "200 OK";
+    static char notmodstr[] = "304 Not Modified";
+    static char authzstr[] = "401 Authorization Required";
+    static char forbiddenstr[] = "403 Forbidden";
+    static char notfoundstr[] = "404 Not Found";
+    static char errorstr[] = "500 Server Error";
+    
+    char* str = nullptr;
+
     switch (status) {
-        case XMLTOOLING_HTTP_STATUS_NOTMODIFIED:    codestr="304 Not Modified"; break;
-        case XMLTOOLING_HTTP_STATUS_UNAUTHORIZED:   codestr="401 Authorization Required"; break;
-        case XMLTOOLING_HTTP_STATUS_FORBIDDEN:      codestr="403 Forbidden"; break;
-        case XMLTOOLING_HTTP_STATUS_NOTFOUND:       codestr="404 Not Found"; break;
-        case XMLTOOLING_HTTP_STATUS_ERROR:          codestr="500 Server Error"; break;
+        case XMLTOOLING_HTTP_STATUS_NOTMODIFIED:    str = notmodstr; break;
+        case XMLTOOLING_HTTP_STATUS_UNAUTHORIZED:   str = authzstr; break;
+        case XMLTOOLING_HTTP_STATUS_FORBIDDEN:      str = forbiddenstr; break;
+        case XMLTOOLING_HTTP_STATUS_NOTFOUND:       str = notfoundstr; break;
+        case XMLTOOLING_HTTP_STATUS_ERROR:          str = errorstr; break;
+
+        default: str = okstr;
     }
-    m_lpECB->ServerSupportFunction(m_lpECB->ConnID, HSE_REQ_SEND_RESPONSE_HEADER, (void*)codestr, 0, (LPDWORD)hdr.c_str());
+    m_lpECB->ServerSupportFunction(m_lpECB->ConnID, HSE_REQ_SEND_RESPONSE_HEADER, str, 0, (LPDWORD)hdr.c_str());
     char buf[1024];
     while (in) {
         in.read(buf,1024);
@@ -931,6 +960,8 @@ public:
     return HSE_STATUS_SUCCESS;
   }
   long sendRedirect(const char* url) {
+    static char _status[] = "302 Moved";
+
     HTTPResponse::sendRedirect(url);
     string hdr=string("Location: ") + url + "\r\n"
       "Content-Type: text/html\r\n"
@@ -940,10 +971,10 @@ public:
     for (multimap<string,string>::const_iterator i = m_headers.begin(); i != m_headers.end(); ++i)
         hdr += i->first + ": " + i->second + "\r\n";
     hdr += "\r\n";
-    m_lpECB->ServerSupportFunction(m_lpECB->ConnID, HSE_REQ_SEND_RESPONSE_HEADER, "302 Moved", 0, (LPDWORD)hdr.c_str());
-    static const char* redmsg="<HTML><BODY>Redirecting...</BODY></HTML>";
+    m_lpECB->ServerSupportFunction(m_lpECB->ConnID, HSE_REQ_SEND_RESPONSE_HEADER, _status, 0, (LPDWORD)hdr.c_str());
+    static char redmsg[] = "<HTML><BODY>Redirecting...</BODY></HTML>";
     DWORD resplen=40;
-    m_lpECB->WriteClient(m_lpECB->ConnID, (LPVOID)redmsg, &resplen, HSE_IO_SYNC);
+    m_lpECB->WriteClient(m_lpECB->ConnID, redmsg, &resplen, HSE_IO_SYNC);
     return HSE_STATUS_SUCCESS;
   }
   // Decline happens in the POST processor if this isn't the handler url
@@ -969,14 +1000,10 @@ public:
 
         if (m_lpECB->ServerSupportFunction(m_lpECB->ConnID, HSE_REQ_GET_CERT_INFO_EX, (LPVOID)&ccex, (LPDWORD)dwSize, nullptr)) {
             if (ccex.CertContext.cbCertEncoded) {
-                xsecsize_t outlen;
+                XMLSize_t outlen;
                 XMLByte* serialized = Base64::encode(reinterpret_cast<XMLByte*>(CertificateBuf), ccex.CertContext.cbCertEncoded, &outlen);
                 m_certs.push_back(reinterpret_cast<char*>(serialized));
-#ifdef SHIBSP_XERCESC_HAS_XMLBYTE_RELEASE
-                XMLString::release(&serialized);
-#else
                 XMLString::release((char**)&serialized);
-#endif
             }
         }
       }
@@ -1001,8 +1028,9 @@ public:
         else
             break;
     }
-    if (bRequired && s.empty())
+    if (bRequired && s.empty()) {
         log(SPRequest::SPError, string("missing required server variable: ") + lpszVariable);
+    }
   }
 };
 
@@ -1021,13 +1049,14 @@ void GetServerVariable(LPEXTENSION_CONTROL_BLOCK lpECB, LPSTR lpszVariable, dyna
             break;
     }
     if (bRequired && s.empty()) {
-        string msg = string("Missing required server variable: ") + lpszVariable;
-        LogEvent(nullptr, EVENTLOG_ERROR_TYPE, 2100, nullptr, msg.c_str());
+        Category::getInstance(SHIBSP_LOGCAT ".ISAPI").error("missing required server variable: %s", lpszVariable);
     }
 }
 
 extern "C" DWORD WINAPI HttpExtensionProc(LPEXTENSION_CONTROL_BLOCK lpECB)
 {
+    static char _INSTANCE_ID[] = "INSTANCE_ID";
+
     try {
         string threadid("[");
         threadid += lexical_cast<string>(getpid()) + "] isapi_shib_extension";
@@ -1035,7 +1064,7 @@ extern "C" DWORD WINAPI HttpExtensionProc(LPEXTENSION_CONTROL_BLOCK lpECB)
 
         // Determine web site number. This can't really fail, I don't think.
         dynabuf buf(128);
-        GetServerVariable(lpECB,"INSTANCE_ID",buf,10);
+        GetServerVariable(lpECB,_INSTANCE_ID,buf,10);
         if (buf.empty())
             return WriteClientError(lpECB, "Shibboleth Extension failed to obtain INSTANCE_ID server variable.");
 
@@ -1051,7 +1080,7 @@ extern "C" DWORD WINAPI HttpExtensionProc(LPEXTENSION_CONTROL_BLOCK lpECB)
         return WriteClientError(lpECB, "Shibboleth Extension failed to process request");
 
     }
-    catch(bad_alloc) {
+    catch(const bad_alloc&) {
         return WriteClientError(lpECB, "Out of Memory");
     }
     catch(long e) {
@@ -1060,12 +1089,12 @@ extern "C" DWORD WINAPI HttpExtensionProc(LPEXTENSION_CONTROL_BLOCK lpECB)
         else
             return WriteClientError(lpECB, "Server detected unexpected IIS error.");
     }
-    catch (std::exception& e) {
-        LogEvent(nullptr, EVENTLOG_ERROR_TYPE, 2100, nullptr, e.what());
-        return WriteClientError(lpECB, "Shibboleth Extension caught an exception, check Event Log for details.");
+    catch (const std::exception& e) {
+        Category::getInstance(SHIBSP_LOGCAT ".ISAPI").error("ISAPI extension caught an exception: %s", e.what());
+        return WriteClientError(lpECB, "Shibboleth Extension caught an exception, check native log for details.");
     }
     catch(...) {
-        LogEvent(nullptr, EVENTLOG_ERROR_TYPE, 2100, nullptr, "Shibboleth Extension threw an unknown exception.");
+        Category::getInstance(SHIBSP_LOGCAT ".ISAPI").crit("ISAPI filter caught an unknown exception");
         if (g_catchAll)
             return WriteClientError(lpECB, "Shibboleth Extension threw an unknown exception.");
         throw;
