@@ -38,6 +38,7 @@
 
 #include <vector>
 #include <fstream>
+#define BOOST_BIND_GLOBAL_PLACEHOLDERS
 #include <boost/bind.hpp>
 #include <boost/lexical_cast.hpp>
 #include <boost/algorithm/string.hpp>
@@ -155,7 +156,7 @@ Handler::~Handler()
 
 #ifndef SHIBSP_LITE
 
-void Handler::generateMetadata(SPSSODescriptor& role, const char* handlerURL) const
+void Handler::generateMetadata(SPSSODescriptor&, const char*) const
 {
 }
 
@@ -186,55 +187,69 @@ void Handler::cleanRelayState(
     const Application& application, const xmltooling::HTTPRequest& request, xmltooling::HTTPResponse& response
     ) const
 {
-    // Only cookie-based relay state requires cleaning.
     pair<bool,const char*> mech = getString("relayState");
     if (!mech.first) {
         // Check for setting on Sessions element.
         const PropertySet* sessionprop = application.getPropertySet("Sessions");
-        if (sessionprop)
+        if (sessionprop) {
             mech = sessionprop->getString("relayState");
-    }
-    if (!mech.first || !mech.second || strncmp(mech.second, "cookie", 6))
-        return;
-    
-    int maxCookies = 25,purgedCookies = 0;
-    mech.second += 6;
-    if (*mech.second == ':' && isdigit(*(++mech.second))) {
-        maxCookies = atoi(mech.second);
-        if (maxCookies == 0)
-            maxCookies = 25;
+        }
     }
 
-    string exp;
+    int maxRSCookies = 20,purgedRSCookies = 0;
+    int maxOSCookies = 20,purgedOSCookies = 0;
 
-    // Walk the list of cookies backwards by name.
-    const map<string,string>& cookies = request.getCookies();
-    for (map<string,string>::const_reverse_iterator i = cookies.rbegin(); i != cookies.rend(); ++i) {
-        // Process relay state cookies only.
-        if (starts_with(i->first, "_shibstate_")) {
-            if (maxCookies > 0) {
-                // Keep it, but count it against the limit.
-                --maxCookies;
-            }
-            else {
-                // We're over the limit, so everything here and older gets cleaned up.
-                if (exp.empty())
-                    exp = string(application.getCookieNameProps("_shibstate_").second) + "; expires=Mon, 01 Jan 2001 00:00:00 GMT";
-                response.setCookie(i->first.c_str(), exp.c_str());
-                ++purgedCookies;
+    if (mech.first && !strncmp(mech.second, "cookie", 6)) {
+        mech.second += 6;
+        if (*mech.second == ':' && isdigit(*(++mech.second))) {
+            maxRSCookies = maxOSCookies = atoi(mech.second);
+            if (maxRSCookies == 0) {
+                maxRSCookies = maxOSCookies = 20;
             }
         }
     }
 
-    if (purgedCookies > 0)
-        log(SPRequest::SPDebug, string("purged ") + lexical_cast<string>(purgedCookies) + " stale relay state cookie(s) from client");
+    // Walk the list of cookies backwards by name.
+    const map<string,string>& cookies = request.getCookies();
+    for (map<string,string>::const_reverse_iterator i = cookies.rbegin(); i != cookies.rend(); ++i) {
+        if (starts_with(i->first, "_shibstate_")) {
+            if (maxRSCookies > 0) {
+                // Keep it, but count it against the limit.
+                --maxRSCookies;
+            }
+            else {
+                // We're over the limit, so everything here and older gets cleaned up.
+                response.setCookie(i->first.c_str(), nullptr, 0, HTTPResponse::SAMESITE_NONE);
+                ++purgedRSCookies;
+            }
+        }
+        else if (starts_with(i->first, "_opensaml_req_")) {
+            if (maxOSCookies > 0) {
+                // Keep it, but count it against the limit.
+                --maxOSCookies;
+            }
+            else {
+                // We're over the limit, so everything here and older gets cleaned up.
+                response.setCookie(i->first.c_str(), nullptr, 0, HTTPResponse::SAMESITE_NONE);
+                ++purgedOSCookies;
+            }
+        }
+    }
+
+    if (purgedRSCookies > 0)
+        log(SPRequest::SPDebug, string("purged ") + lexical_cast<string>(purgedRSCookies) + " stale relay state cookie(s) from client");
+    if (purgedOSCookies > 0)
+        log(SPRequest::SPDebug, string("purged ") + lexical_cast<string>(purgedOSCookies) + " stale request correlation cookie(s) from client");
 }
 
 void Handler::preserveRelayState(const Application& application, HTTPResponse& response, string& relayState) const
 {
-    // The empty string implies no state to deal with.
-    if (relayState.empty())
+    // The empty string implies no state to deal with but we need to generate a correlation handle.
+    if (relayState.empty()) {
+        generateRandomHex(relayState, 4);
+        relayState = "corr:" + lexical_cast<string>(time(nullptr)) + '_' + relayState;
         return;
+    }
 
     // No setting means just pass state by value.
     pair<bool,const char*> mech = getString("relayState");
@@ -251,14 +266,14 @@ void Handler::preserveRelayState(const Application& application, HTTPResponse& r
         // Here we store the state in a cookie and send a fixed
         // value so we can recognize it on the way back.
         if (relayState.find("cookie:") != 0 && relayState.find("ss:") != 0) {
-            pair<string,const char*> shib_cookie = application.getCookieNameProps("_shibstate_");
-            string stateval = XMLToolingConfig::getConfig().getURLEncoder()->encode(relayState.c_str()) + shib_cookie.second;
             // Generate a random key for the cookie name instead of the fixed name.
             string rsKey;
             generateRandomHex(rsKey, 4);
             rsKey = lexical_cast<string>(time(nullptr)) + '_' + rsKey;
-            shib_cookie.first = "_shibstate_" + rsKey;
-            response.setCookie(shib_cookie.first.c_str(), stateval.c_str());
+            string shib_cookie_name = "_shibstate_" + rsKey;
+            response.setCookie(shib_cookie_name.c_str(),
+                XMLToolingConfig::getConfig().getURLEncoder()->encode(relayState.c_str()).c_str(),
+                0, HTTPResponse::SAMESITE_NONE);
             relayState = "cookie:" + rsKey;
         }
     }
@@ -318,6 +333,12 @@ void Handler::recoverRelayState(
 {
     SPConfig& conf = SPConfig::getConfig();
 
+    // Sentry value that signifies it was only a correlation tool.
+    if (starts_with(relayState, "corr:")) {
+        relayState.clear();
+        return;
+    }
+
     // Look for StorageService-backed state of the form "ss:SSID:key".
     const char* state = relayState.c_str();
     if (strstr(state,"ss:") == state) {
@@ -383,9 +404,8 @@ void Handler::recoverRelayState(
         state += 7;
         if (*state) {
             // Pull the value from the "relay state" cookie.
-            pair<string,const char*> relay_cookie = application.getCookieNameProps("_shibstate_");
-            relay_cookie.first = string("_shibstate_") + state;
-            state = request.getCookie(relay_cookie.first.c_str());
+            string relay_cookie = string("_shibstate_") + state;
+            state = request.getCookie(relay_cookie.c_str());
             if (state && *state) {
                 // URL-decode the value.
                 char* rscopy = strdup(state);
@@ -393,9 +413,7 @@ void Handler::recoverRelayState(
                 relayState = rscopy;
                 free(rscopy);
                 if (clear) {
-                    string exp(relay_cookie.second);
-                    exp += "; expires=Mon, 01 Jan 2001 00:00:00 GMT";
-                    response.setCookie(relay_cookie.first.c_str(), exp.c_str());
+                    response.setCookie(relay_cookie.c_str(), nullptr, 0, HTTPResponse::SAMESITE_NONE);
                 }
                 request.absolutize(relayState);
                 return;
@@ -594,7 +612,7 @@ void AbstractHandler::preservePostData(
 #endif
 
     // No specs mean no save.
-    const PropertySet* props=application.getPropertySet("Sessions");
+    const PropertySet* props = application.getPropertySet("Sessions");
     pair<bool,const char*> mech = props ? props->getString("postData") : pair<bool,const char*>(false,nullptr);
     if (!mech.first) {
         m_log.info("postData property not supplied, form data will not be preserved across SSO");
@@ -646,11 +664,10 @@ void AbstractHandler::preservePostData(
             postkey = string(mech.second-3) + ':' + out.string();
         }
 
-        pair<string,const char*> shib_cookie = getPostCookieNameProps(application, relayState);
+        string shib_cookie = getPostCookieName(application, relayState);
 
         // Purge any cookies in excess of 25.
-        int maxCookies = 25,purgedCookies = 0;
-        string exp;
+        int maxCookies = 20,purgedCookies = 0;
 
         // Walk the list of cookies backwards by name.
         const map<string,string>& cookies = request.getCookies();
@@ -663,9 +680,7 @@ void AbstractHandler::preservePostData(
                 }
                 else {
                     // We're over the limit, so everything here and older gets cleaned up.
-                    if (exp.empty())
-                        exp = string(shib_cookie.second) + "; expires=Mon, 01 Jan 2001 00:00:00 GMT";
-                    response.setCookie(i->first.c_str(), exp.c_str());
+                    response.setCookie(i->first.c_str(), nullptr, 0, HTTPResponse::SAMESITE_NONE);
                     ++purgedCookies;
                 }
             }
@@ -675,8 +690,7 @@ void AbstractHandler::preservePostData(
             log(SPRequest::SPDebug, string("purged ") + lexical_cast<string>(purgedCookies) + " stale POST preservation cookie(s) from client");
 
         // Set a cookie with key info.
-        postkey += shib_cookie.second;
-        response.setCookie(shib_cookie.first.c_str(), postkey.c_str());
+        response.setCookie(shib_cookie.c_str(), postkey.c_str(), 0, HTTPResponse::SAMESITE_NONE);
     }
     else {
         postData.destroy();
@@ -688,16 +702,15 @@ DDF AbstractHandler::recoverPostData(
     const Application& application, const HTTPRequest& request, HTTPResponse& response, const char* relayState
     ) const
 {
+    string shib_cookie = getPostCookieName(application, relayState);
+
     // First we need the post recovery cookie.
-    pair<string,const char*> shib_cookie = getPostCookieNameProps(application, relayState);
-    const char* cookie = request.getCookie(shib_cookie.first.c_str());
+    const char* cookie = request.getCookie(shib_cookie.c_str());
     if (!cookie || !*cookie)
         return DDF();
 
     // Clear the cookie.
-    string exp(shib_cookie.second);
-    exp += "; expires=Mon, 01 Jan 2001 00:00:00 GMT";
-    response.setCookie(shib_cookie.first.c_str(), exp.c_str());
+    response.setCookie(shib_cookie.c_str(), nullptr, 0, HTTPResponse::SAMESITE_NONE);
 
     // Look for StorageService-backed state of the form "ss:SSID:key".
     const char* state = cookie;
@@ -786,21 +799,20 @@ long AbstractHandler::sendPostResponse(
     return httpResponse.sendResponse(str);
 }
 
-pair<string,const char*> AbstractHandler::getPostCookieNameProps(const Application& app, const char* relayState) const
+string AbstractHandler::getPostCookieName(const Application& app, const char* relayState) const
 {
     // Decorates the name of the cookie with the relay state key, if any.
     // Doing so gives a better assurance that the recovered data really
     // belongs to the relayed request.
-    pair<string,const char*> shib_cookie=app.getCookieNameProps("_shibpost_");
     if (strstr(relayState, "cookie:") == relayState) {
-        shib_cookie.first = string("_shibpost_") + (relayState + 7);
+        return string("_shibpost_") + (relayState + 7);
     }
     else if (strstr(relayState, "ss:") == relayState) {
         const char* pch = strchr(relayState + 3, ':');
         if (pch)
-            shib_cookie.first = string("_shibpost_") + (pch + 1);
+            return string("_shibpost_") + (pch + 1);
     }
-    return shib_cookie;
+    return app.getCookieName("_shibpost_");
 }
 
 DDF AbstractHandler::getPostData(const Application& application, const HTTPRequest& request) const
